
# q-expansion of newform 819.2.j.e, downloaded from the LMFDB on 10 February 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 819
weight = 2
poly_data = [1, 0, 0, -1, 0, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[92, [1, 0, 0, 0, 0, 0]], [703, [0, 0, 0, -1, 0, 0]], [379, [1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 1, -1, 0, -1, 1], [0, 0, 0, 0, 0, 0], [0, 1, -1, 1, -1, 1], [0, -2, 0, 0, -1, 0], [-1, 0, 1, 1, -1, -1], [1, 0, 0, 0, 0, 0], [3, -1, 0, -3, 1, 1], [0, -2, 2, -1, 2, -2], [0, -4, -1, 0, -1, -4], [-2, -4, -4, 0, 5, -1], [-4, 1, 1, 4, -2, -2], [0, 0, 1, 1, 1, 0], [0, 0, 0, 0, 3, -3], [-2, 3, 3, 0, -2, -1], [0, -4, 0, 2, 0, -4], [-1, 0, -1, 1, 1, 1], [-1, 9, -1, 1, -8, -8], [0, -8, 5, -5, 5, -8], [3, 0, -5, -3, 5, 5], [-7, 6, 6, 0, -1, -5], [0, -7, 1, 0, 6, 6], [0, -1, -5, 8, -5, -1], [4, -1, -1, 0, 1, 0], [0, -1, 4, 5, 4, -1], [7, 1, 1, 0, 4, -5], [13, 1, -1, -13, 0, 0], [0, 0, 2, -11, 2, 0], [0, -2, 1, -6, 1, -2], [-1, 4, 7, 1, -11, -11], [-6, -1, -1, 0, -10, 11], [-3, 1, 1, 0, -7, 6], [0, -4, 11, 1, 11, -4], [10, -1, 0, -10, 1, 1], [14, 2, 2, 0, -5, 3], [0, 4, 2, 14, 2, 4], [-4, 8, -4, 4, -4, -4], [-5, -1, 9, 5, -8, -8], [0, 7, -5, 13, -5, 7], [3, 6, 6, 0, -8, 2], [0, -4, 12, -1, 12, -4], [-2, 12, 3, 2, -15, -15], [4, 3, 3, 0, 11, -14], [0, -7, 13, 3, 13, -7], [-1, -10, 1, 1, 9, 9], [-10, -4, -4, 0, 1, 3], [8, -4, 3, -8, 1, 1], [-7, -8, -8, 0, -4, 12], [-2, -8, -8, 0, 13, -5], [0, -4, 10, 0, -6, -6], [0, 4, 4, 7, 4, 4], [0, 8, -3, 3, -3, 8], [3, -1, -1, 0, -10, 11], [-3, 1, 11, 3, -12, -12], [1, 12, 12, 0, -5, -7], [0, -7, -12, 10, -12, -7], [7, 1, -4, -7, 3, 3], [11, 6, -11, -11, 5, 5], [0, 5, -6, 2, -6, 5], [6, -9, 2, -6, 7, 7], [-4, 13, 13, 0, -4, -9], [-1, 2, -17, 1, 15, 15], [-12, -10, -10, 0, 11, -1], [-13, 11, 11, 0, -4, -7], [0, 2, -1, 0, -1, -1], [0, 4, -17, 6, -17, 4], [0, -2, -3, -3, -3, -2], [0, 13, -18, -7, -18, 13], [-15, 0, 0, 0, -3, 3], [3, 14, -7, -3, -7, -7], [13, 1, 1, 0, 1, -2], [6, 1, -9, -6, 8, 8], [0, 3, 10, -13, 10, 3], [7, 23, -17, -7, -6, -6], [0, 8, -2, -15, -2, 8], [-1, -13, -13, 0, 22, -9], [0, 6, -7, -6, -7, 6], [0, 2, -5, 0, 3, 3], [0, -1, 13, 11, 13, -1], [0, 2, 10, 11, 10, 2], [2, -3, 0, -2, 3, 3], [1, -12, -12, 0, 14, -2], [-9, -10, -10, 0, 18, -8], [10, 1, -1, -10, 0, 0], [-4, 3, 3, 0, -14, 11], [0, -15, -4, 2, -4, -15], [0, 5, -10, -10, -10, 5], [10, -21, -21, 0, 11, 10], [0, 1, -15, -7, -15, 1], [-10, 1, 1, 0, -5, 4], [-7, 5, 5, 0, 0, -5], [0, -13, 3, -13, 3, -13], [-15, -17, 4, 15, 13, 13], [-5, 8, 6, 5, -14, -14], [-4, 3, 3, 0, 17, -20], [0, 16, -4, 14, -4, 16], [-1, -13, -13, 0, 20, -7], [0, 14, -28, -5, -28, 14], [9, -7, 20, -9, -13, -13], [0, 16, -4, -6, -4, 16], [0, 12, 0, -15, 0, 12], [-1, -3, -3, 0, -5, 8], [17, -6, 20, -17, -14, -14], [4, 8, -9, -4, 1, 1], [0, 15, -21, -5, -21, 15], [-15, -17, 9, 15, 8, 8], [-12, -4, -5, 12, 9, 9], [19, 5, 5, 0, 7, -12], [0, 13, 7, 7, 7, 13], [22, -1, 7, -22, -6, -6], [12, -12, -12, 0, 2, 10], [0, 14, 1, 8, 1, 14], [-2, -3, 8, 2, -5, -5], [0, 12, 12, 0, -23, 11], [2, 13, 15, -2, -28, -28], [2, -3, -3, 0, 12, -9], [-23, -13, 10, 23, 3, 3], [21, 1, 1, 0, 2, -3], [2, -7, -3, -2, 10, 10], [0, 13, -25, -4, -25, 13], [-22, 0, 0, 0, 6, -6], [4, -2, -16, -4, 18, 18], [1, 10, 10, 0, 10, -20], [0, 1, 4, 13, 4, 1], [21, -7, -10, -21, 17, 17], [0, 2, -5, -5, -5, 2], [-2, 8, 8, 0, 8, -16], [0, -23, 9, -24, 9, -23], [0, -2, 3, 25, 3, -2], [7, 10, 10, 0, -15, 5], [0, 30, -22, 7, -22, 30], [7, -13, 17, -7, -4, -4], [-21, 1, 1, 0, -14, 13], [0, -22, 18, 9, 18, -22], [-18, -6, -6, 0, 8, -2], [0, 12, 9, 17, 9, 12], [40, 3, 3, 0, -2, -1], [10, -21, -9, -10, 30, 30], [32, 11, -10, -32, -1, -1], [48, 1, 1, 0, 2, -3], [-8, -9, -17, 8, 26, 26], [-31, -5, -5, 0, -5, 10], [0, 0, 1, 19, 1, 0], [-15, 1, 16, 15, -17, -17], [6, -3, -3, 0, -11, 14], [-29, 8, -1, 29, -7, -7], [-26, -6, -6, 0, -14, 20], [-7, -18, -18, 0, 35, -17], [0, -19, 24, 0, -5, -5], [0, -1, 8, -25, 8, -1], [0, 12, -20, -33, -20, 12], [0, 5, -14, -28, -14, 5], [12, -15, -15, 0, -8, 23], [-9, 5, 5, 0, 1, -6], [0, 15, -3, 15, -3, 15], [24, 23, -10, -24, -13, -13], [19, 4, 4, 0, 13, -17], [0, -9, 5, 1, 5, -9], [0, 12, 4, 23, 4, 12], [-25, 4, 4, 0, 13, -17], [39, 1, -9, -39, 8, 8], [0, 24, -14, 11, -14, 24], [-6, 20, 20, 0, -10, -10], [16, 14, 14, 0, 4, -18], [0, -22, 4, -11, 4, -22], [-31, 7, -3, 31, -4, -4], [-38, 7, 1, 38, -8, -8], [12, 12, -26, -12, 14, 14], [3, 5, -27, -3, 22, 22]]
